@@ -63,7 +63,8 @@ class ImageTool(object):
             'black': [0, 0, 0],
         }
 
-    def locate(self, template_path, template_resize=1.0, img_path=None, locate_center=True, threshold_value=None, as_gray=False,
+    def locate(self, template_path, template_resize=1.0, img_path=None, locate_center=True, threshold_value=None,
+               as_gray=False,
                as_binary=False, img_shape_times=1.0, return_score_only=False):
         if threshold_value:
             self.threshold_value = threshold_value
@@ -101,8 +102,8 @@ class ImageTool(object):
             return center if locate_center else [int(x), int(y)]
         else:
             print(f"[ {green(tell_the_datetime())} ]\n "
-                  f"    matching image: [ {blue(template_path or 'ScreenShot')} ]\n "
-                  f"    using template: [ {blue(img_path)} ]\n "
+                  f"    matching image: [ {blue(img_path or 'ScreenShot')} ]\n "
+                  f"    using template: [ {blue(template_path)} ]\n "
                   f"    >>> score not pass! score: {hred(score)}\n")
 
     def patch_locate(self, template_path_list, template_resize=1.0, img_path=None, locate_center=True,
@@ -340,8 +341,6 @@ class FlowTool(object):
         self.base_path = os.path.split(os.path.abspath(__file__))[0]
         self.default_chrome_icon = os.path.join(self.base_path, "resource/icons/chrome_icon.png")
         self.screen_width, self.screen_height = pyautogui.size()
-        # self.db_ms = Missions(table_name=self.project_name)
-        # self.db_ms.createTable()
         self.ms_dic = dict()
         self.total_steps = 0
         self._ready_steps()
@@ -357,26 +356,27 @@ class FlowTool(object):
             "SaveWithVim": self._save_data_with_vim,
             "TermCommand": self._save_data_with_vim,
             "Click": self._mouse_click,
+            "HotKey": self._hot_key,
+            "InputABC": self._input_abc,
         }
 
     def _ready_steps(self):
-        # count = 0
-        self.ms_dic = {self.operate_list.index(i)+1: i for i in self.operate_list}
-        # for dic in self.operate_list:
-        #     count += 1
-        #     self.db_ms.name = dic.get("name")
-        #     self.db_ms.method = dic.get("method")
-        #     self.db_ms.step = count
-        #     self.db_ms.params = json.dumps(dic, ensure_ascii=True)
-        #     self.db_ms.create_time = tell_the_datetime()
+        print("steps: ")
+        count = 1
+        for step_data in self.operate_list:
+            self.ms_dic[count] = step_data
+            print(f"    [ {green(count)} ] -- [ {green(step_data.get('name'))} ]")
+            count += 1
         self.total_steps = len(self.operate_list)
 
     def _search_and_click(self, params):
         icon_path = params.get("icon_path")
         match_options = params.get("match_options")
         speed = params.get("speed") or "fast"
+        not_locate = params.get("not_locate") or "exit"  # "exit", "next1", "jump1"
         pre_delay = params.get("pre_delay") or 0
         sub_delay = params.get("sub_delay") or 1
+        jump_step = re.findall(r'\d+', not_locate)
 
         match_options = match_options if isinstance(match_options, dict) else self.default_match_opt
         time.sleep(int(pre_delay))
@@ -387,17 +387,31 @@ class FlowTool(object):
             as_binary=match_options.get('as_binary'),
             img_shape_times=match_options.get('img_shape_times'),
         )
-        delay = self._speed(speed)
-        self._delay_move(*icon_position, delay=delay)
-        pyautogui.click()
-        time.sleep(sub_delay)
+        if icon_position:
+            delay = self._speed(speed)
+            self._delay_move(*icon_position, delay=delay)
+            pyautogui.click()
+            time.sleep(sub_delay)
+            return {'next': int(params.get('cur_step', 1)) + 1, "pack": {"position": icon_position}}
+        else:
+            if not_locate.lower() == "exit":
+                print(f"System exit because can not locate template: \n    {icon_path}")
+                raise KeyboardInterrupt
+            elif "jump" in not_locate.lower():
+                jump_step = jump_step[0] if jump_step else 0
+                return {'next': jump_step}
+            else:
+                jump_step = jump_step[0] if jump_step else 1
+                return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
 
     def _multi_search_and_click(self, params):
         icon_paths = params.get("icon_paths")
         match_options = params.get("match_options")
+        not_locate = params.get("not_locate") or "next1"    # jump1
         speed = params.get("speed") or "fast"
         pre_delay = params.get("pre_delay") or 0
         sub_delay = params.get("sub_delay") or 1
+        jump_step = re.findall(r'\d+', not_locate)
 
         match_options = match_options if isinstance(match_options, dict) else self.default_match_opt
         time.sleep(int(pre_delay))
@@ -408,17 +422,32 @@ class FlowTool(object):
             as_binary=match_options.get('as_binary'),
             img_shape_times=match_options.get('img_shape_times'),
         )
-        delay = self._speed(speed)
-        self._delay_move(*icon_position, delay=delay)
-        pyautogui.click()
-        time.sleep(sub_delay)
+        if icon_position:
+            delay = self._speed(speed)
+            self._delay_move(*icon_position, delay=delay)
+            pyautogui.click()
+            time.sleep(sub_delay)
+            return {'next': int(params.get('cur_step', 1)) + 1, "pack": {"position": icon_position}}
+        else:
+            not_locate = not_locate.lower()
+            if not_locate == "exit":
+                print(f"System exit because can not locate template: \n    {icon_paths}")
+                raise KeyboardInterrupt
+            elif "jump" in not_locate.lower():
+                jump_step = jump_step[0] if jump_step else 0
+                return {'next': jump_step}
+            else:
+                jump_step = jump_step[0] if jump_step else 1
+                return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
 
     def _open_chrome_and_enter_url(self, params):
         url = params.get("url")
+        not_locate = params.get("not_locate") or "next1"    # jump1
         chrome_icon = params.get("chrome_icon")
         speed = params.get("speed") or "fast"
         pre_delay = params.get("pre_delay") or 0
         sub_delay = params.get("sub_delay") or 5
+        jump_step = re.findall(r'\d+', not_locate)
 
         time.sleep(pre_delay)
         if not chrome_icon or not os.path.exists(chrome_icon):
@@ -427,14 +456,27 @@ class FlowTool(object):
             template_path=chrome_icon,
             as_gray=True,
         )
-        self._delay_move(*chrome_position)
-        time.sleep(0.1)
-        pyautogui.click()
-        time.sleep(0.3)
-        pyautogui.hotkey('ctrl', 'l')
-        self._delay_write(url, self._speed(speed))
-        pyautogui.press('enter')
-        time.sleep(sub_delay)
+        if chrome_position:
+            self._delay_move(*chrome_position)
+            time.sleep(0.1)
+            pyautogui.click()
+            time.sleep(0.3)
+            pyautogui.hotkey('ctrl', 'l')
+            self._delay_write(url, self._speed(speed))
+            pyautogui.press('enter')
+            time.sleep(sub_delay)
+            return {'next': int(params.get('cur_step', 1)) + 1, "pack": {"position": chrome_position}}
+        else:
+            not_locate = not_locate.lower()
+            if not_locate == "exit":
+                print(f"System exit because can not locate chrome icon: \n    {chrome_icon}")
+                raise KeyboardInterrupt
+            elif "jump" in not_locate.lower():
+                jump_step = jump_step[0] if jump_step else 0
+                return {'next': jump_step}
+            else:
+                jump_step = jump_step[0] if jump_step else 1
+                return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
 
     def _wait_icon_show(self, params):
         """
@@ -454,18 +496,15 @@ class FlowTool(object):
         icon_path = params.get("icon_path")
         match_options = params.get("match_options")
         interval = int(params.get("interval")) or 1
-        after_showed = params.get("after_showed") or "NextStep"
+        after_showed = params.get("after_showed") or "next1"
         time_out = int(params.get("time_out")) or 120
-        if_timeout = params.get("if_timeout") or "End"
+        if_timeout = params.get("if_timeout") or "exit"
 
         match_options = match_options if isinstance(match_options, dict) else self.default_match_opt
         show_sta = False
         times_start = time.time()
         icon_position = [0, 0]
-        count = 0
         while True:
-            if count > 1 and count % 10 == 0:
-                print(f"icon still not show: \n  {icon_path}")
             if time.time() - times_start > time_out:
                 break
             icon_position = self.it.locate(
@@ -479,28 +518,32 @@ class FlowTool(object):
                 show_sta = True
                 break
             time.sleep(interval)
-            count += 1
+        jump_step = re.findall(r'\d+', after_showed)
+
         if show_sta:
-            if after_showed == "NextStep":
-                return True
-            elif after_showed == "ReturnPosition":
-                return icon_position
+            jump_step = jump_step[0] if jump_step else 1
+            r_dic = {'next': int(params.get('cur_step', 1)) + int(jump_step), 'pack': {'position': icon_position}}
+            return r_dic
         else:
-            if if_timeout == 'End':
-                print(red("\nSys out because icon not found!\n"))
+            if_timeout = if_timeout.lower()
+            if if_timeout == 'exit':
+                print(red("\nSys out because icon not found!"))
                 print(f"    [ {red(icon_path)} ]\n    [ {tell_the_datetime()} ]")
-            elif if_timeout == 'NextStep':
-                return True
-            elif 'JumpToStep' in if_timeout:
-                return re.findall(r"\d+", if_timeout)[0]
+                raise KeyboardInterrupt
+            elif "jump" in if_timeout.lower():
+                jump_step = jump_step[0] if jump_step else 0
+                return {'next': jump_step}
+            else:
+                jump_step = jump_step[0] if jump_step else 1
+                return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
 
     def _wait_icon_gone(self, params):
         icon_path = params.get("icon_path")
         match_options = params.get("match_options")
         interval = int(params.get("interval")) or 1
-        after_gone = params.get("after_gone") or "NextStep"
+        after_gone = params.get("after_gone") or "next1"
         time_out = int(params.get("time_out")) or 120
-        if_timeout = params.get("if_timeout") or "End"
+        if_timeout = params.get("if_timeout") or "exit"
 
         match_options = match_options if isinstance(match_options, dict) else self.default_match_opt
         gone_sta = False
@@ -524,24 +567,28 @@ class FlowTool(object):
                 break
             time.sleep(interval)
             count += 1
+        jump_step = re.findall(r'\d+', after_gone)
         if gone_sta:
-            if after_gone == "NextStep":
-                return True
-            elif after_gone == "ReturnFinalPosition":
-                return icon_position
+            jump_step = jump_step[0] if jump_step else 1
+            return {'next': int(params.get('cur_step', 1)) + int(jump_step), 'pack': {'position': icon_position}}
         else:
-            if if_timeout == 'End':
-                print(red("\nSys out because icon Gone!\n"))
+            if_timeout = if_timeout.lower()
+            if if_timeout == 'exit':
+                print(red("\nSys out because timeout when waiting icon gone!"))
                 print(f"    [ {red(icon_path)} ]\n    [ {tell_the_datetime()} ]")
-            elif if_timeout == 'NextStep':
-                return True
-            elif 'JumpToStep' in if_timeout:
-                return re.findall(r"\d+", if_timeout)[0]
+                raise KeyboardInterrupt
+            elif "jump" in if_timeout.lower():
+                jump_step = jump_step[0] if jump_step else 0
+                return {'next': jump_step}
+            else:
+                jump_step = jump_step[0] if jump_step else 1
+                return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
 
     def _save_data_with_vim(self, params):
         file_full_path = params.get("file_full_path")
         pre_delay = params.get("pre_delay") or 0
         sub_delay = params.get("sub_delay") or 1
+        after = params.get("after") or 'next1'
 
         time.sleep(pre_delay)
         pyautogui.hotkey('ctrl', 'alt', 't')
@@ -576,9 +623,18 @@ class FlowTool(object):
         pyautogui.hotkey('ctrl', 'shift', 'q')
         time.sleep(sub_delay)
 
+        jump_step = re.findall(r'\d+', after.lower())
+        if 'next' in after:
+            jump_step = jump_step[0] if jump_step else 1
+            return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
+        else:
+            jump_step = jump_step[0] if jump_step else 0
+            return {'next': jump_step}
+
     def _terminal_operations(self, params):
         cmd = params.get("cmd")
         root_password = params.get("root_password")
+        after = params.get("after") or 'next1'
 
         pyautogui.hotkey('ctrl', 'alt', 't')
         time.sleep(0.7)
@@ -586,8 +642,8 @@ class FlowTool(object):
         time.sleep(0.3)
         pyautogui.press('enter')
         if self.it.locate(
-            template_path=os.path.join(self.base_path, 'resource/icons/terminal_input_password.png'),
-            as_gray=True,
+                template_path=os.path.join(self.base_path, 'resource/icons/terminal_input_password.png'),
+                as_gray=True,
         ):
             if root_password:
                 self._delay_write(f"{root_password}", 0.01)
@@ -595,23 +651,66 @@ class FlowTool(object):
                 pyautogui.press('enter')
             else:
                 print("please input password!")
-                self._wait_icon_gone(
-                    icon_path=os.path.join(self.base_path, 'resource/icons/terminal_input_password.png'),
-                    match_options={'as_gray': True},
-                    time_out=1000000,
+                self._wait_icon_gone({
+                    "icon_path": os.path.join(self.base_path, 'resource/icons/terminal_input_password.png'),
+                    "match_options": {'as_gray': True},
+                    "time_out": 1000000}
                 )
 
+        jump_step = re.findall(r'\d+', after.lower())
+        if 'next' in after:
+            jump_step = jump_step[0] if jump_step else 1
+            return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
+        else:
+            jump_step = jump_step[0] if jump_step else 0
+            return {'next': jump_step}
+
+    def _hot_key(self, params):
+        pre_delay = params.get("pre_delay") or 0
+        sub_delay = params.get("sub_delay") or 1
+        after = params.get("after") or 'next1'
+        time.sleep(pre_delay)
+        key_list = params.get('key_list')
+        if len(key_list) > 1:
+            pyautogui.hotkey(*key_list)
+        else:
+            pyautogui.press(*key_list)
+        time.sleep(sub_delay)
+
+        jump_step = re.findall(r'\d+', after.lower())
+        if 'next' in after:
+            jump_step = jump_step[0] if jump_step else 1
+            return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
+        else:
+            jump_step = jump_step[0] if jump_step else 0
+            return {'next': jump_step}
+
+    def _input_abc(self, params):
+        pre_delay = params.get("pre_delay") or 0
+        sub_delay = params.get("sub_delay") or 1
+        after = params.get("after") or 'next1'
+        time.sleep(pre_delay)
+        words = params.get('words')
+        self._delay_write(f"{words}", 0.01)
+        time.sleep(sub_delay)
+
+        jump_step = re.findall(r'\d+', after.lower())
+        if 'next' in after:
+            jump_step = jump_step[0] if jump_step else 1
+            return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
+        else:
+            jump_step = jump_step[0] if jump_step else 0
+            return {'next': jump_step}
+
     def _mouse_click(self, params):
+        """
+        position: TopLeft",  #  "center", "TopRight", "BottomLeft", "BottomRight", or [1000, 1000],
+        :return:
+        """
         position = params.get("position")
         pre_delay = int(params.get("pre_delay")) or 0
         sub_delay = int(params.get("sub_delay")) or 1
-        """
-        position: TopLeft",  #  "center", "TopRight", "BottomLeft", "BottomRight", or [1000, 1000],
-        :param position:
-        :param pre_delay:
-        :param sub_delay:
-        :return:
-        """
+        after = params.get("after") or 'next1'
         time.sleep(pre_delay)
         if position == 'TopLeft':
             self._delay_move(1, 1)
@@ -628,6 +727,46 @@ class FlowTool(object):
         pyautogui.click()
         time.sleep(sub_delay)
 
+        jump_step = re.findall(r'\d+', after.lower())
+        if 'next' in after:
+            jump_step = jump_step[0] if jump_step else 1
+            return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
+        else:
+            jump_step = jump_step[0] if jump_step else 0
+            return {'next': jump_step}
+
+    def _mouse_drag(self, params):
+        """
+        position: TopLeft",  #  "center", "TopRight", "BottomLeft", "BottomRight", or [1000, 1000],
+        :return:
+        """
+        position = params.get("position")
+        pre_delay = int(params.get("pre_delay")) or 0
+        sub_delay = int(params.get("sub_delay")) or 1
+        after = params.get("after") or 'next1'
+        time.sleep(pre_delay)
+        if position == 'TopLeft':
+            self._delay_move(1, 1)
+        elif position == 'TopRight':
+            self._delay_move(self.screen_width - 1, 1)
+        elif position == 'center':
+            self._delay_move(int(self.screen_width / 2), int(self.screen_height / 2))
+        elif position == 'BottomLeft':
+            self._delay_move(0, self.screen_height - 1)
+        elif position == 'BottomRight':
+            self._delay_move(self.screen_width + 1, self.screen_height - 1)
+        elif isinstance(position, list):
+            self._delay_drag(*position)
+        time.sleep(sub_delay)
+
+        jump_step = re.findall(r'\d+', after.lower())
+        if 'next' in after:
+            jump_step = jump_step[0] if jump_step else 1
+            return {'next': int(params.get('cur_step', 1)) + int(jump_step)}
+        else:
+            jump_step = jump_step[0] if jump_step else 0
+            return {'next': jump_step}
+
     @staticmethod
     def _speed(speed):
         if isinstance(speed, int) or isinstance(speed, float):
@@ -653,14 +792,27 @@ class FlowTool(object):
         pyautogui.write(words, interval=delay_for_each)
 
     def start(self):
-        for i in range(1, self.total_steps + 1):
-            # step = self.db_ms.selectOne(SQL().WHERE('step', '=', i))
-            step = self.ms_dic.get(i)
-            name = step.get("name")
-            method = step.get("method")
-            params = step
-            print(f"running step: [ {hgreen(i)} ] -- [ {name} ]")
-            self.methods.get(method)(params=params)
+        step = 1
+        pre_pack = {}
+        try:
+            while True:
+                step_data = self.ms_dic.get(step)
+                if step_data:
+                    name = step_data.get("name")
+                    method = step_data.get("method")
+                    params = step_data
+                    params['cur_step'] = step
+                    if pre_pack:
+                        params.update(pre_pack)
+                    print(f"running step: [ {hgreen(step)} ] -- [ {name} ]")
+                    run_result = self.methods.get(method)(params=params)
+                    step = run_result.get('next')
+                    pre_pack = run_result.get("pack")
+                else:
+                    print("all process done!")
+                    break
+        except KeyboardInterrupt:
+            print(red(f"[ {tell_the_datetime()} ] sys exit!"))
 
 
 def load_mission_from_json(jf_path):
@@ -688,7 +840,7 @@ def start_missions():
     json_file = args.json_file
     loop = args.loop
     start_time = args.start_time or tell_the_datetime()
-    end_time = args.end_time or tell_the_datetime(time_stamp=(time.time()+3600*24*365*10000))
+    end_time = args.end_time or tell_the_datetime(time_stamp=(time.time() + 3600 * 24 * 365 * 10000))
 
     if not os.path.exists(json_file):
         print(hred(f"File Not Exists!\n    {json_file}"))
